package com.yunzhi.shigongli.common;

import com.yunzhi.shigongli.vo.AMapWeather;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;

@Slf4j
@Component
public class Weather {

    @Value("${amap.weather.api}")
    String api;

    @Value("${amap.weather.backImage}")
    private String backImage;

    // 默认支持的背景图片
    private String[] imageList = new String[] {
            "中雨",
            "中雪",
            "冰雹",
            "冻雨",
            "多云",
            "夜晚多云",
            "夜晚扬沙",
            "夜晚晴",
            "夜晚沙尘暴",
            "夜晚阵雨",
            "夜晚降雪",
            "夜晚雾",
            "大雨",
            "大雪",
            "大雾",
            "大风",
            "小雨",
            "小雪",
            "无数据",
            "晴",
            "暴雨",
            "暴雪",
            "沙尘暴",
            "阴",
            "阵雨",
            "阵雪",
            "雨夹雪",
            "雷阵雨",
            "雷阵雨伴有冰雹",
            "雾"
    };

    public AMapWeather getWeather(String areaId) throws Exception {
        RestTemplate restTemplate = new RestTemplate();

        String url = String.format(api, areaId);

        log.debug("请求高德天气接口: {}", url);

        ResponseEntity<AMapWeather> response = restTemplate.exchange(url, HttpMethod.GET, null, AMapWeather.class);
        if (null == response) {
            throw new Exception("请求天气接口异常");
        }

        HttpStatus statusCode = response.getStatusCode();
        boolean isSuccessful = statusCode.is2xxSuccessful();
        if (!isSuccessful) {
            log.error("请求天气接口[{}]出错, [{}]", url, statusCode.toString());
            throw new Exception("请求天气接口失败: " + statusCode.getReasonPhrase());
        }

        AMapWeather result = response.getBody();

        log.debug("高德天气接口返回: {}", result.toString());

        if (!"1".equals(result.getStatus())) {
            log.error("高德天气接口错误: {}", result.toString());
            throw new Exception("请求天气接口错误: " + result.getInfocode());
        }

        return result;
    }

    public String getBackgroundImage(String weatherName) {
        boolean isExist = Arrays.stream(imageList).anyMatch(x -> x.equals(weatherName));
        String imageName = isExist ? weatherName : "无数据";
        return  String.format(backImage, imageName);
    }
}
