package com.yunzhi.shigongli.common;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.api.impl.WxMaServiceImpl;
import cn.binarywang.wx.miniapp.bean.WxMaCodeLineColor;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import cn.binarywang.wx.miniapp.config.impl.WxMaDefaultConfigImpl;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.service.WxPayService;
import com.github.binarywang.wxpay.service.impl.WxPayServiceImpl;
import com.yunzhi.shigongli.config.WeixinConfig;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateMessage;
import me.chanjar.weixin.mp.config.WxMpConfigStorage;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class WxUtils {
    private static WxMaService service;
    private static WxPayService payService;
    // 公众号
    private WxMpService mpService;

    WeixinConfig config;

    @Autowired
    public WxUtils(WeixinConfig config) {
        this.config = config;
        setService();
        setPay();
    }

    private void setService() {
        // 小程序
        WeixinConfig.Miniapp miniapp = config.getMiniapp();
        WxMaDefaultConfigImpl conf = new WxMaDefaultConfigImpl();
        conf.setAppid(miniapp.getAppid());
        conf.setSecret(miniapp.getSecret());
        conf.setToken(miniapp.getToken());
        conf.setAesKey(miniapp.getAesKey());
        conf.setMsgDataFormat(miniapp.getMsgDataFormat());
        service = new WxMaServiceImpl();
        service.setWxMaConfig(conf);

        // 公众号
        WeixinConfig.Mp mp = config.getMp();
        WxMpDefaultConfigImpl wxMpDefaultConfig = new WxMpDefaultConfigImpl();
        wxMpDefaultConfig.setAppId(mp.getAppid());
        wxMpDefaultConfig.setSecret(mp.getSecret());
        wxMpDefaultConfig.setToken(mp.getToken());
        wxMpDefaultConfig.setAesKey(mp.getAesKey());
        mpService = new WxMpServiceImpl();
        mpService.setWxMpConfigStorage(wxMpDefaultConfig);
    }

    private void setPay() {
        WxPayConfig payConfig = new WxPayConfig();
        payConfig.setAppId(config.getPay().getAppId());
        payConfig.setMchId(config.getPay().getMchId());
        payConfig.setMchKey(config.getPay().getMchKey());
        payConfig.setSubAppId(config.getPay().getSubAppId());
        payConfig.setSubMchId(config.getPay().getSubMchId());
        payConfig.setKeyPath(config.getPay().getKeyPath());
        // 固定小程序支付
        payConfig.setTradeType(WxPayConstants.TradeType.JSAPI);
        // 可以指定是否使用沙箱环境
        payConfig.setUseSandboxEnv(false);

        payService = new WxPayServiceImpl();
        payService.setConfig(payConfig);
    }

    public WxMaService getService() {
        return service;
    }
    public WxMpService getMpService() {
        return mpService;
    }
    public WxPayService getPayService() {
        return payService;
    }

    public WeixinConfig.Miniapp getMiniappConf() {
        return config.getMiniapp();
    }
    public WeixinConfig.Mp getMpConf() {
        return config.getMp();
    }

    @Data
    public static class AQrCodeParam {
        private String scene;
        private String page;
        private int width;
        private boolean autoColor;
        private WxMaCodeLineColor lineColor;
        private boolean isHyaline;
    }

    /**
     * 发送小程序订阅消息
     * @param templateId
     * @param openid
     * @param page
     * @param dataList
     * @throws Exception
     */
    @Async
    public void sendMessage(String templateId, String openid, String page, List<WxMaSubscribeMessage.Data> dataList) throws Exception {
        WxMaSubscribeMessage wxMaSubscribeMessage = WxMaSubscribeMessage
                .builder()
                .templateId(templateId)
                .toUser(openid)
                .page(page)
                .data(dataList)
                .build();

        try {
            service.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);
        } catch (Exception e) {
            e.printStackTrace();

            // 再发送一次
            Thread.sleep(1000);
            service.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);
        }
    }

    /**
     * 发送公众号模板消息
     * @param toUser
     * @param templateId
     * @param miniAppId
     * @param page
     * @param datas
     * @return
     * @throws WxErrorException
     */
    public void sendMpTemplateMessage(String toUser, String templateId, String miniAppId, String page, String url, List<WxMpTemplateData> datas) throws Exception {
        String pagepath = StringUtils.trim(page, "/");
        WxMpTemplateMessage.MiniProgram miniProgram = null == miniAppId ? null : new WxMpTemplateMessage.MiniProgram(miniAppId, pagepath, false);

        log.info("模板信息-小程序内容: {}", miniProgram);

        WxMpTemplateMessage templateMessage = WxMpTemplateMessage.builder()
                .toUser(toUser)
                .miniProgram(miniProgram)
                .templateId(templateId)
                .url(url)
                .build();

        for (WxMpTemplateData data: datas) {
            templateMessage.addData(data);
        }

        try {
            mpService.getTemplateMsgService().sendTemplateMsg(templateMessage);
        } catch (Exception e) {
            e.printStackTrace();

            // 再发送一次
            Thread.sleep(1000);
            mpService.getTemplateMsgService().sendTemplateMsg(templateMessage);
        }
    }
}
