package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.BaseController;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.ResponseBean;
import com.yunzhi.shigongli.entity.TaImage;
import com.yunzhi.shigongli.entity.TaNote;
import com.yunzhi.shigongli.entity.TaNoteResource;
import com.yunzhi.shigongli.service.ITaImageService;
import com.yunzhi.shigongli.service.ITaNoteResourceService;
import com.yunzhi.shigongli.service.ITaNoteService;
import com.yunzhi.shigongli.vo.NoteVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
    * 笔记 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "笔记")
@RestController
@RequestMapping("/")
public class TaNoteController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaNoteController.class);

    @Autowired
    public ITaNoteService iTaNoteService;

    @Autowired
    public ITaImageService iTaImageService;

    @Autowired
    public ITaNoteResourceService iTaNoteResourceService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/wx/note",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = TaNote.class)
    public ResponseBean getWxList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                  @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                  @ApiParam("定位地点") @RequestParam(value ="location", required = false) String location,
                                  @ApiParam("查询内容") @RequestParam(value ="q", required = false) String q) throws Exception{

        IPage<TaNote> pg = new Page<>(pageNum, pageSize);

        IPage<TaNote> result = iTaNoteService.getWxPageBy(pg, getCurrentPerson(), location, q);
        return ResponseBean.success(result);
    }

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/note",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = TaNote.class)
    public ResponseBean noteList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                 @ApiParam("笔记类型") @RequestParam(value ="noteType", required = false) String noteType,
                                 @ApiParam("简介") @RequestParam(value ="summary", required = false) String summary) throws Exception{

		    IPage<TaNote> pg = new Page<>(pageNum, pageSize);

            IPage<TaNote> result = iTaNoteService.getPageBy(pg, noteType, summary);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param noteVO 实体对象
     * @return
     */
    @RequestMapping(value="/admin/note",method= RequestMethod.POST)
    @ApiOperation(value="保存笔记", notes = "保存笔记", httpMethod = "POST", response = TaNote.class)
    public ResponseBean saveNote(@ApiParam("保存内容") @RequestBody NoteVO noteVO) throws Exception{
        if (null == noteVO) {
            return ResponseBean.error("没有找到保存内容", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (null == noteVO.getNoteType()) {
            return ResponseBean.error("请设置笔记类型", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // banner 图集
        List<TaImage> imageList = noteVO.getImageList();
        if ((null == imageList || imageList.size() == 0) && Constants.NOTE_TYPE_IMAGE.equals(noteVO.getNoteType())) {
            return ResponseBean.error("请上传 banner 图片", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 关联地点
        List<TaNoteResource> resourceList = noteVO.getResourceList();
        if (null == resourceList || resourceList.size() == 0) {
            // 暂时不处理
        }

        LocalDateTime now = LocalDateTime.now();
        noteVO.setCreateDate(now);
        if (null == noteVO.getStatus()) {
            noteVO.setStatus(Constants.STATUS_NORMAL);
        }

        TaNote taNote = new TaNote();
        BeanUtils.copyProperties(noteVO, taNote);

        if (iTaNoteService.save(taNote)) {
            noteVO.setNoteId(taNote.getNoteId());

            // banner 图集
            if (Constants.NOTE_TYPE_IMAGE.equals(noteVO.getNoteType())) {
                for (TaImage image : imageList) {
                    image.setCreateDate(now);
                    image.setImageType("banner");
                }
                iTaImageService.updateImages(Constants.TARGET_NOTE, taNote.getNoteId(), imageList);
            }

            // 关联地点
            if (null != resourceList && resourceList.size() > 0) {
                iTaNoteResourceService.updateResource(taNote.getNoteId(), resourceList);
            }

            return ResponseBean.success(noteVO);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/note/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除笔记", notes = "删除笔记", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean deleteNote(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaNoteService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param noteVO 实体对象
     * @return
     */
    @RequestMapping(value="/admin/note/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新笔记", notes = "更新笔记", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean updateNote(@ApiParam("对象ID") @PathVariable String id,
                                   @ApiParam("更新内容") @RequestBody NoteVO noteVO) throws Exception {

        TaNote taNote = iTaNoteService.getById(id);
        if (null == taNote || Constants.STATUS_DELETED == taNote.getStatus()) {
            return ResponseBean.error("没有找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }


        if (null == noteVO.getNoteType()) {
            return ResponseBean.error("请设置笔记类型", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // banner 图集
        List<TaImage> imageList = noteVO.getImageList();
        if ((null == imageList || imageList.size() == 0) && Constants.NOTE_TYPE_IMAGE.equals(noteVO.getNoteType())) {
            return ResponseBean.error("请上传 banner 图片", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 关联地点
        List<TaNoteResource> resourceList = noteVO.getResourceList();
        if (null == resourceList || resourceList.size() == 0) {
            // 暂时不处理
        }

        LocalDateTime now = LocalDateTime.now();
        BeanUtils.copyProperties(noteVO, taNote);
        taNote.setNoteId(id);

        if (iTaNoteService.updateById(taNote)) {

            // banner 图集
            if (Constants.NOTE_TYPE_IMAGE.equals(noteVO.getNoteType())) {
                for (TaImage image : imageList) {
                    image.setCreateDate(now);
                    image.setImageType("banner");
                }
                iTaImageService.updateImages(Constants.TARGET_NOTE, taNote.getNoteId(), imageList);
            }

            // 关联地点
            if (null != resourceList && resourceList.size() > 0) {
                iTaNoteResourceService.updateResource(taNote.getNoteId(), resourceList);
            }

            return ResponseBean.success(noteVO);
        } else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/note/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getAdminDetail(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaNote taNote = iTaNoteService.getById(id);
        if (null == taNote || Constants.STATUS_DELETED == taNote.getStatus()) {
            return ResponseBean.error("没有找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        NoteVO noteVO = new NoteVO();
        BeanUtils.copyProperties(taNote, noteVO);


        noteVO.setImageList(iTaImageService.getListBy(Constants.TARGET_NOTE, id));
        noteVO.setResourceList(iTaNoteResourceService.getListBy(id));

        return ResponseBean.success(noteVO);
    }


    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/note/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxDetail(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaNote taNote = iTaNoteService.getById(id);
        if (null == taNote || Constants.STATUS_DELETED == taNote.getStatus()) {
            return ResponseBean.error("没有找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        NoteVO noteVO = new NoteVO();
        BeanUtils.copyProperties(taNote, noteVO);


        noteVO.setImageList(iTaImageService.getListBy(Constants.TARGET_NOTE, id));
        noteVO.setWxResourceList(iTaNoteResourceService.getResourceListBy(id, getCurrentPerson().getPersonId()));

        return ResponseBean.success(noteVO);
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/note/{id}/publish/{statusCode}",method= RequestMethod.PUT)
    @ApiOperation(value="发布/取消发布", notes = "发布/取消发布", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taPvGet(@ApiParam("对象ID") @PathVariable String id,
                                @ApiParam(value = "发布状态", allowableValues = "on,off") @PathVariable String statusCode) throws Exception {
        TaNote taNote = iTaNoteService.getById(id);
        if (null == taNote || Constants.STATUS_DELETED == taNote.getStatus()) {
            return ResponseBean.error("没有找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (!"on".equals(statusCode) && !"off".equals(statusCode)) {
            return ResponseBean.error("发布状态不正确", ResponseBean.ERROR_UNAVAILABLE);
        }

        Integer status = "on".equals(statusCode) ? 1 : 0;

        taNote.setStatus(status);

        iTaNoteService.updateById(taNote);

        return ResponseBean.success(null);
    }
}
