package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.TaOrder;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.service.ITaOrderPayService;
import com.yunzhi.shigongli.service.ITaOrderService;
import com.yunzhi.shigongli.service.ITaVerifyTargetService;
import com.yunzhi.shigongli.vo.MineOrder;
import com.yunzhi.shigongli.vo.MineOrderSummary;
import com.yunzhi.shigongli.vo.OrderSubListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.shigongli.service.ITaOrderSubService;
import com.yunzhi.shigongli.entity.TaOrderSub;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
    * 子订单表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "子订单表")
@RestController
@RequestMapping("/")
public class TaOrderSubController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrderSubController.class);

    @Autowired
    public ITaOrderSubService iTaOrderSubService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaOrderPayService iTaOrderPayService;

    @Autowired
    public ITaVerifyTargetService iTaVerifyTargetService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/taOrderSub",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = OrderSubListVO.class)
    public ResponseBean taOrderSubList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("商家名称") @RequestParam(value = "shopName", required = false) String shopName,
                                       @ApiParam("标题") @RequestParam(value = "description", required = false) String description,
                                       @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                       @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end,
                                       @ApiParam(value = "订单编号") @RequestParam(value = "orderId", required = false) String orderId,
                                       @ApiParam(value = "订单状态") @RequestParam(value = "status", required = false) Integer status,
                                       @ApiParam(value = "手机号") @RequestParam(value = "phone", required = false) String phone) throws Exception{

        IPage<OrderSubListVO> pg = new Page<>(pageNum, pageSize);

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }


        IPage<OrderSubListVO> result = iTaOrderSubService.getAdminPagedBy(pg, shopName, description, startDate, endDate, orderId, status, phone);
        return ResponseBean.success(result);
    }

    /**
     * 订单列表导出
     * @return
     */
    @RequestMapping(value="/admin/taOrderSub/export",method= RequestMethod.GET)
    @ApiOperation(value="订单列表导出", notes = "订单列表导出", httpMethod = "GET", response = OrderSubListVO.class)
    public void exportTaOrderSubList(@ApiParam("商家名称") @RequestParam(value = "shopName", required = false) String shopName,
                                     @ApiParam("标题") @RequestParam(value = "description", required = false) String description,
                                     @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                     @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end,
                                     @ApiParam(value = "订单编号") @RequestParam(value = "orderId", required = false) String orderId,
                                     @ApiParam(value = "订单状态") @RequestParam(value = "status", required = false) Integer status,
                                     @ApiParam(value = "手机号") @RequestParam(value = "phone", required = false) String phone,
                                     HttpServletResponse response) throws Exception{

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }


        IPage<OrderSubListVO> result = iTaOrderSubService.getAdminPagedBy(null, shopName, description, startDate, endDate, orderId, status, phone);
        List<OrderSubListVO> list = result.getRecords();
        if (null != list) {
            for(OrderSubListVO item : list) {
                if (null != item.getIsVerified() && item.getIsVerified()) {
                    // excelutils 会把 100 转换为已核销
                    item.setStatus(100);
                }
            }
        }

        ExcelUtils.flush(response, OrderSubListVO.class, list, "订单列表");
    }

    /**
     * 订单项目发起支付
     * @param id
     * @param request
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/wx/orderSub/{id}/pay",method= RequestMethod.POST)
    @ApiOperation(value="订单项目发起支付", notes = "订单项目发起支付", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taOrderPay(@ApiParam("订单项目ID") @PathVariable String id,
                                   HttpServletRequest request) throws Exception{
        TaOrderSub taOrderSub = iTaOrderSubService.getById(id);
        if (null == taOrderSub || Constants.STATUS_DELETED == taOrderSub.getStatus()) {
            return ResponseBean.error("未找到订单信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaOrder taOrder = iTaOrderService.getById(taOrderSub.getOrderId());
        if (null == taOrder || Constants.STATUS_DELETED == taOrder.getStatus()) {
            return ResponseBean.error("未找到订单信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (Constants.ORDER_READY != taOrderSub.getStatus()) {
            return ResponseBean.error("订单非未支付状态, 不能完成操作", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaPerson person = getCurrentPerson();
        WxPayMpOrderResult result = iTaOrderPayService.prePay(person, taOrder, taOrderSub);
        return ResponseBean.success(result);
    }

    /**
     * 订单项目退款
     * @param id
     * @return
     * @throws Exception
     */
    @DeleteMapping("/wx/orderSub/{id}/refund")
    @ApiOperation(value="订单项目退款", notes = "订单项目退款", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean refund(@ApiParam(value = "订单项目ID") @PathVariable String id) throws Exception {
        TaOrderSub taOrderSub = iTaOrderSubService.getById(id);
        if (null == taOrderSub || Constants.STATUS_DELETED == taOrderSub.getStatus()) {
            return ResponseBean.error("未找到退款项目", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaOrder taOrder = iTaOrderService.getById(taOrderSub.getOrderId());
        if (null == taOrder || Constants.STATUS_DELETED == taOrder.getStatus()) {
            return ResponseBean.error("未找到当前订单", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (Constants.ORDER_PAYED != taOrderSub.getStatus()) {
            return ResponseBean.error("当前项目未支付", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaPerson taPerson = getCurrentPerson();

        if (!taPerson.getPersonId().equals(taOrder.getPersonId())) {
            return ResponseBean.error("您无权操作当前订单项目", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 如果存在已核销的, 则不能退款
        int cnt = iTaVerifyTargetService.getCountBy(taOrderSub.getOrderId(), taOrderSub.getSubOrderId(), 1);
        if (cnt > 0) {
            return ResponseBean.error("当前存在已核销内容, 不能退款", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        iTaOrderPayService.refundOrderSub(taPerson, taOrder, taOrderSub);
        iTaOrderSubService.refund(taPerson, taOrder, taOrderSub);
        // 删除所有待核销内容
        iTaVerifyTargetService.removeBy(taOrderSub.getOrderId(), taOrderSub.getSubOrderId());

        return ResponseBean.success("操作完成");
    }


    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/orderSub/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taOrderSubGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{

        TaPerson taPerson = getCurrentPerson();
        MineOrder result =  iTaOrderSubService.getSingleOrderSubOf(taPerson, id);

        return ResponseBean.success(result);
    }

    @GetMapping("/wx/orderSub")
    @ApiOperation(value="订单列表", notes = "订单列表", httpMethod = "GET", response = MineOrder.class)
    public ResponseBean getWxMyOrders(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("订单ID") @RequestParam(value ="orderId", required = false) String orderId,
                                      @ApiParam("状态") @RequestParam(value ="status", required = false) Integer status,
                                      @ApiParam("是否核销") @RequestParam(value ="isVerified", required = false) Integer isVerified) {
        TaPerson taPerson = getCurrentPerson();

        IPage<MineOrder> pg = new Page<>(pageNum, pageSize);
        IPage<MineOrder> result = iTaOrderSubService.getOrderSubListOf(pg, taPerson, status, orderId, isVerified);

        return ResponseBean.success(result);
    }

    @GetMapping("/wx/orderSub/summary/mine")
    @ApiOperation(value="我的订单概况", notes = "我的订单概况", httpMethod = "GET", response = MineOrder.class)
    public ResponseBean getWxMyOrders() {
        TaPerson taPerson = getCurrentPerson();

        MineOrderSummary result = iTaOrderSubService.getOrderSummary(taPerson);

        return ResponseBean.success(result);
    }
}
