package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.entity.TaTravel;
import com.yunzhi.shigongli.entity.TaTravelItem;
import com.yunzhi.shigongli.entity.TaTravelPerson;
import com.yunzhi.shigongli.service.ITaPvService;
import com.yunzhi.shigongli.service.ITaTravelItemService;
import com.yunzhi.shigongli.service.ITaTravelPersonService;
import com.yunzhi.shigongli.service.ITaTravelService;
import com.yunzhi.shigongli.vo.TravelDataVO;
import com.yunzhi.shigongli.vo.TravelItemDataVO;
import com.yunzhi.shigongli.vo.TravelListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.ibatis.annotations.Delete;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
    * 行程表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "行程表")
@RestController
@RequestMapping("/")
public class TaTravelController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaTravelController.class);

    @Autowired
    public ITaTravelService iTaTravelService;

    @Autowired
    public ITaTravelItemService iTaTravelItemService;

    @Autowired
    public ITaTravelPersonService iTaTravelPersonService;


    /**
     * 获取行程列表
     * @return
     * @throws Exception
     */
    @GetMapping("/admin/travel")
    @ApiOperation(value="获取行程列表", notes = "获取行程列表", httpMethod = "GET", response = TravelDataVO.class)
    public ResponseBean getAdminTravel(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone,
                                       @ApiParam(value = "起始时间", example = "2021-10-01") @RequestParam(value ="start", required = false) String start,
                                       @ApiParam(value = "结束时间", example = "2021-10-31") @RequestParam(value ="end", required = false) String end) throws Exception {
        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }

        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<TravelListVO> pg = new Page<>(pageNum, pageSize);

        IPage<TravelListVO> result = iTaTravelService.getAdminList(pg, phone, startDate, endDate);

        return ResponseBean.success(result);
    }

    /**
     * 命定行程列表
     * @return
     * @throws Exception
     */
    @GetMapping("/admin/travel/destiny")
    @ApiOperation(value="ADMIN命定行程列表", notes = "ADMIN命定行程列表", httpMethod = "GET", response = TravelDataVO.class)
    public ResponseBean getAdminDestinyTravel(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                              @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                              @ApiParam("行程描述") @RequestParam(value ="title", required = false) String title,
                                              @ApiParam("状态") @RequestParam(value ="status", required = false) Integer status) throws Exception {
        IPage<TaTravel> pg = new Page<>(pageNum, pageSize);

        IPage<TaTravel> travelList = iTaTravelService.getAdminDestinyList(pg, title, status);

        return ResponseBean.success(travelList);
    }


    /**
     * 行程详情
     * @return
     * @throws Exception
     */
    @GetMapping("/admin/travel/{id}")
    @ApiOperation(value="行程详情", notes = "行程详情", httpMethod = "GET", response = TravelDataVO.class)
    public ResponseBean getTravelDetail(@ApiParam("行程ID") @PathVariable String id) throws Exception {
        TaTravel taTravel = iTaTravelService.getById(id);
        if (null == taTravel || taTravel.getStatus() == Constants.STATUS_DELETED) {
            return ResponseBean.error("没有找到行程信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TravelDataVO travelDataVO = new TravelDataVO();
        BeanUtils.copyProperties(taTravel, travelDataVO);

        List<TravelItemDataVO> travelItemDetails = iTaTravelItemService.getTravelItemDetails(id);
        travelDataVO.setTravelItemList(travelItemDetails);

        return ResponseBean.success(travelDataVO);
    }

    /**
     * 获取命定行程
     * @return
     * @throws Exception
     */
    @GetMapping("/wx/travel/destiny")
    @ApiOperation(value="获取命定行程", notes = "获取命定行程", httpMethod = "GET", response = TravelDataVO.class)
    public ResponseBean getDestinyTravel() throws Exception {
        TaPerson taPerson = getCurrentPerson();

        TravelDataVO travelData = iTaTravelPersonService.getDestinyTravel(taPerson.getPersonId());
        if (null == travelData) {
            return ResponseBean.success(null);
        }

        List<TravelItemDataVO> itemDataVOList = iTaTravelItemService.getTravelItemDetails(travelData.getTravelId());

        travelData.setTravelItemList(itemDataVOList);

        return ResponseBean.success(travelData);
    }

    /**
     * 保存命定行程
     * @return
     * @throws Exception
     */
    @PostMapping("/wx/travel/destiny")
    @ApiOperation(value="保存命定行程", notes = "保存命定行程", httpMethod = "POST", response = TravelDataVO.class)
    public ResponseBean saveDestinyTravel() throws Exception {
        TaPerson taPerson = getCurrentPerson();

        TaTravel travel = iTaTravelPersonService.getAndCopyDestinyTravel(taPerson.getPersonId());
        TravelDataVO travelDataVO = new TravelDataVO();
        BeanUtils.copyProperties(travel, travelDataVO);

        List<TravelItemDataVO> itemDataVOList = iTaTravelItemService.getTravelItemDetails(travel.getTravelId());

        travelDataVO.setTravelItemList(itemDataVOList);

        return ResponseBean.success(travel);
    }


    /**
     * 保存命定行程
     * @return
     * @throws Exception
     */
    @PostMapping("/admin/travel/destiny")
    @ApiOperation(value="保存命定行程", notes = "保存命定行程", httpMethod = "POST", response = TravelDataVO.class)
    public ResponseBean saveDestinyTravel(@ApiParam("保存内容") @RequestBody TravelDataVO travelDataVO) throws Exception {
        if (null == travelDataVO) {
            return ResponseBean.error("没有找到保存内容", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(travelDataVO.getTitle())) {
            return ResponseBean.error("请填写命定行程描述", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        List<TravelItemDataVO> travelItemList = travelDataVO.getTravelItemList();
        if (null == travelItemList || travelItemList.size() < 1) {
            return ResponseBean.error("请维护相关地点", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaTravel taTravel = new TaTravel();
        BeanUtils.copyProperties(travelDataVO, taTravel);
        taTravel.setDayNum(1);
        taTravel.setIsDestiny(true);
        taTravel.setIsSystem(true);

        if (taTravel.getStatus() == null) {
            taTravel.setStatus(Constants.STATUS_READY);
        }
        if (taTravel.getWeight() == null) {
            taTravel.setWeight(0);
        }

        // 保存行程主记录
        if (iTaTravelService.save(taTravel)) {
            // 获取最新的记录
            travelDataVO.setTravelItemList(iTaTravelItemService.updateTravelItems(taTravel.getTravelId(), travelItemList));
            travelDataVO.setTravelId(taTravel.getTravelId());

            return ResponseBean.success(travelDataVO);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 更新行程
     * @return
     * @throws Exception
     */
    @PutMapping("/admin/travel/{id}")
    @ApiOperation(value="更新行程", notes = "更新行程", httpMethod = "PUT", response = TravelDataVO.class)
    public ResponseBean updateDestinyTravel(@ApiParam("行程ID") @PathVariable String id,
                                            @ApiParam("更新内容") @RequestBody TravelDataVO travelDataVO) throws Exception {
        if (null == travelDataVO) {
            return ResponseBean.error("没有找到更新内容", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaTravel origin = iTaTravelService.getById(id);
        if (null == origin || origin.getStatus() == Constants.STATUS_DELETED) {
            return ResponseBean.error("没有找到行程信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        travelDataVO.setTravelId(id);

        if (StringUtils.isEmpty(travelDataVO.getTitle())) {
            return ResponseBean.error("请填写行程描述", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        List<TravelItemDataVO> travelItemList = travelDataVO.getTravelItemList();
        if (null == travelItemList || travelItemList.size() < 1) {
            return ResponseBean.error("请维护相关地点", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaTravel taTravel = new TaTravel();
        BeanUtils.copyProperties(travelDataVO, taTravel);
//        taTravel.setDayNum(1);
//        taTravel.setIsDestiny(true);
//        taTravel.setIsSystem(true);

        if (taTravel.getStatus() == null) {
            taTravel.setStatus(Constants.STATUS_READY);
        }
        if (taTravel.getWeight() == null) {
            taTravel.setWeight(0);
        }

        // 保存行程主记录
        if (iTaTravelService.updateById(taTravel)) {
            // 获取最新的记录
            travelDataVO.setTravelItemList(iTaTravelItemService.updateTravelItems(taTravel.getTravelId(), travelItemList));
            travelDataVO.setTravelId(taTravel.getTravelId());

            return ResponseBean.success(travelDataVO);
        } else {
            return ResponseBean.error("更新失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


    /**
     * 获取我的行程
     * @return
     * @throws Exception
     */
    @GetMapping("/wx/travel/mine")
    @ApiOperation(value="获取我的行程", notes = "获取我的行程", httpMethod = "GET", response = TravelDataVO.class)
    public ResponseBean getMineTravel(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("简易列表") @RequestParam(value ="simple",defaultValue = "false") Boolean simple) throws Exception {
        TaPerson taPerson = getCurrentPerson();

        IPage<TravelDataVO> pg = new Page<>(pageNum, pageSize);

        IPage<TravelDataVO> result = iTaTravelService.getPersonAllList(pg, taPerson.getPersonId(), simple);

        return ResponseBean.success(result);
    }


    /**
     * 保存我的行程
     * @param taTravel
     * @return
     * @throws Exception
     */
    @PostMapping("/wx/travel")
    @ApiOperation(value="保存我的行程", notes = "保存我的行程", httpMethod = "POST", response = TaTravel.class)
    public ResponseBean saveTravel(@ApiParam("保存内容") @RequestBody TaTravel taTravel) throws Exception {
        taTravel.setCreateDate(LocalDateTime.now());
        taTravel.setStatus(Constants.STATUS_NORMAL);
        taTravel.setIsSystem(false);
        taTravel.setIsDestiny(false);
        if (null == taTravel.getDayNum() || taTravel.getDayNum() < 1) {
            taTravel.setDayNum(1);
        }

        if (iTaTravelService.save(taTravel)) {
            TaTravelPerson taTravelPerson = new TaTravelPerson();
            taTravelPerson.setTravelId(taTravel.getTravelId());
            taTravelPerson.setPersonId(getCurrentPerson().getPersonId());
            taTravelPerson.setIsHidden(false);

            if (iTaTravelPersonService.save(taTravelPerson)) {
                return ResponseBean.success(taTravel);
            } else {
                return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
            }

        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 更新我的行程
     * @param id
     * @return
     * @throws Exception
     */
    @PutMapping("/wx/travel/{id}")
    @ApiOperation(value="更新我的行程", notes = "更新我的行程", httpMethod = "PUT", response = TaTravel.class)
    public ResponseBean updateTravel(@ApiParam("行程ID") @PathVariable String id,
                                     @ApiParam("保存内容") @RequestBody TaTravel taTravel) throws Exception {
        TaTravelPerson taTravelPerson = iTaTravelPersonService.getByPerson(getCurrentPerson().getPersonId(), id);
        if (null == taTravelPerson) {
            return ResponseBean.error("未找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaTravel origin = iTaTravelService.getById(id);

        // 只允许修改名称 - 天数
        origin.setTitle(taTravel.getTitle());
        origin.setDayNum(taTravel.getDayNum());

        if (iTaTravelService.updateById(origin)) {
            return ResponseBean.success(origin);
        } else {
            return ResponseBean.error("更新失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 删除我的行程
     * @param id
     * @return
     * @throws Exception
     */
    @DeleteMapping("/wx/travel/{id}")
    @ApiOperation(value="删除我的行程", notes = "删除我的行程", httpMethod = "DELETE", response = TaTravel.class)
    public ResponseBean deleteTravel(@ApiParam("行程ID") @PathVariable String id) throws Exception {
        TaTravelPerson taTravelPerson = iTaTravelPersonService.getByPerson(getCurrentPerson().getPersonId(), id);
        if (null == taTravelPerson) {
            return ResponseBean.error("未找到删除内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (iTaTravelPersonService.removeById(taTravelPerson.getSerialNo())) {
            TaTravel taTravel = iTaTravelService.getById(id);
            if (null != taTravel) {
                iTaTravelService.removeLogicById(taTravel);
            }

            return ResponseBean.success(null);
        } else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 查看我的行程
     * @param id
     * @return
     * @throws Exception
     */
    @GetMapping("/wx/travel/{id}")
    @ApiOperation(value="我的行程详情", notes = "我的行程详情", httpMethod = "GET", response = TaTravel.class)
    public ResponseBean getMineTravel(@ApiParam("行程ID") @PathVariable String id) throws Exception {
        TaTravelPerson taTravelPerson = iTaTravelPersonService.getByPerson(getCurrentPerson().getPersonId(), id);
        if (null == taTravelPerson) {
            return ResponseBean.error("未找到行程内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaTravel origin = iTaTravelService.getById(id);
        if (null == origin || origin.getStatus() == Constants.STATUS_DELETED) {
            return ResponseBean.error("没有找到行程信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TravelDataVO travelDataVO = new TravelDataVO();
        BeanUtils.copyProperties(origin, travelDataVO);

        List<TravelItemDataVO> travelItemDetails = iTaTravelItemService.getTravelItemDetails(id);
        travelDataVO.setTravelItemList(travelItemDetails);

        return ResponseBean.success(travelDataVO);
    }

}
