package com.yunzhi.shigongli.entity;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.baomidou.mybatisplus.annotation.IdType;
import java.time.LocalDateTime;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import java.io.Serializable;

import com.yunzhi.shigongli.common.excelConverter.LocalDateTimeConverter;
import com.yunzhi.shigongli.common.excelConverter.PersonRoleConverter;
import com.yunzhi.shigongli.common.excelConverter.RecommendTypeConverter;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 * 小程序人员表
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value="TaPerson对象", description="小程序人员表")
public class TaPerson implements Serializable {

    @ExcelIgnore
    private static final long serialVersionUID = 1L;

    @ExcelIgnore
    @ApiModelProperty(value = "人员ID")
    @TableId(value = "person_id", type = IdType.UUID)
    private String personId;

    @ExcelIgnore
    @ApiModelProperty(value = "openid")
    private String openid;

    @ExcelIgnore
    @ApiModelProperty(value = "公众号openid")
    private String mpOpenid;

    @ExcelIgnore
    @ApiModelProperty(value = "unionid")
    private String unionid;

    @ExcelIgnore
    @ApiModelProperty(value = "性别")
    private Integer sex;

    @ExcelProperty(value = "用户昵称", index = 0)
    @ColumnWidth(20)
    @ApiModelProperty(value = "昵称")
    private String nickName;

    @ExcelIgnore
    @ApiModelProperty(value = "头像")
    private String avatar;

    @ExcelProperty(value = "手机号码", index = 1)
    @ColumnWidth(20)
    @ApiModelProperty(value = "手机")
    private String phone;

    @ExcelProperty(value = "用户类型", converter = PersonRoleConverter.class, index = 2)
    @ColumnWidth(20)
    @ApiModelProperty(value = "当前角色")
    private String personRole;

    @ExcelIgnore
    @ApiModelProperty(value = "状态")
    private Integer status;

    @ExcelProperty(value = "注册时间", converter = LocalDateTimeConverter.class, index = 7)
    @ColumnWidth(30)
    @ApiModelProperty(value = "创建时间")
    private LocalDateTime createDate;

    @ExcelProperty(value = "用户来源", converter = RecommendTypeConverter.class, index = 3)
    @ColumnWidth(20)
    @ApiModelProperty(value = "来源类型")
    private String fromType;

    @ExcelIgnore
    @ApiModelProperty(value = "推广ID")
    private String recommender;

    @ExcelProperty(value = "来源归属", index = 4)
    @ColumnWidth(30)
    @TableField(exist = false)
    @ApiModelProperty(value = "来源归属")
    private String recommenderName;

    @ExcelProperty(value = "当前绑定", converter = RecommendTypeConverter.class, index = 5)
    @ColumnWidth(20)
    @ApiModelProperty(value = "当前绑定")
    @TableField(exist = false)
    private String currentFrom;

    @ExcelProperty(value = "当前归属", index = 6)
    @ColumnWidth(30)
    @TableField(exist = false)
    @ApiModelProperty(value = "当前归属")
    private String currentRecommenderName;

}
