package com.yunzhi.shigongli.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.common.WxUtils;
import com.yunzhi.shigongli.config.WeixinConfig;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaHotelWorkerService;

import com.yunzhi.shigongli.vo.HotelWorkerListVO;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 民宿员工 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Slf4j
@Service
public class TaHotelWorkerServiceImpl extends BaseServiceImpl<TaHotelWorkerMapper, TaHotelWorker> implements ITaHotelWorkerService {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    TaHotelWorkerMapper hotelWorkerMapper;

    @Autowired
    TaHotelMapper taHotelMapper;

    @Autowired
    TaRoomMapper taRoomMapper;

    @Autowired
    TaMiniappMessageTplMapper messageTplMapper;

    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    TaMpSettingMapper taMpSettingMapper;

    @Autowired
    TaRoomOrderPersonMapper roomOrderPersonMapper;

    @Override
    public IPage<HotelWorkerListVO> getPagedListBy(IPage<HotelWorkerListVO> pg, String hotelId) {
        return hotelWorkerMapper.getPagedListBy(pg, hotelId);
    }

    @Override
    public List<TaHotelWorker> getByPhone(String phone) {
        LambdaQueryWrapper<TaHotelWorker> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaHotelWorker::getPhone, phone);
        queryWrapper.gt(TaHotelWorker::getStatus, Constants.STATUS_DELETED);

        return list(queryWrapper);
    }

    @Override
    public void updateByPerson(TaPerson taPerson) throws Exception {
        hotelWorkerMapper.updateByPerson(taPerson.getPersonId(), taPerson.getPhone());
    }

    @Override
    public void sendMessageBy(TaRoomOrder taRoomOrder) {
        TaHotel taHotel = taHotelMapper.selectById(taRoomOrder.getHotelId());
        List<TaHotelWorker> workerList = getListByHotel(taHotel.getHotelId());
        if (null == workerList || workerList.size() < 1) {
            return;
        }

        TaRoom taRoom = taRoomMapper.selectById(taRoomOrder.getRoomId());
        if (null == taRoom) {
            return;
        }

        TaRoomOrderPerson roomOrderPerson = null;
        List<TaRoomOrderPerson> roomOrderPersonList = roomOrderPersonMapper.getByOrderId(taRoomOrder.getRoomOrderId());
        if (null == roomOrderPersonList || roomOrderPersonList.size() < 1) {
            return;
        } else {
            roomOrderPerson = roomOrderPersonList.get(0);
        }

        String roomName = taRoom.getRoomName();
        if (null != roomName && roomName.length() > 10) {
            roomName = roomName.substring(0, 10) + "...";
        }
        String searchParams = "roomId=" + taRoom.getRoomId()
                + "&hotelId=" + taRoom.getHotelId()
                + "&roomName=" + StringUtils.encodeURIComponent(taRoom.getRoomName());

        // 公众号通知模板
        WeixinConfig.Mp mpConf = wxUtils.getMpConf();
        TaMpSetting taMpSetting = taMpSettingMapper.getByAppAndCode(mpConf.getAppid(), Constants.MP_SETTING_HOTEL_CHECK_IN_TPL);
        String mpTplId = null;
        String mpUrl = null;
        if (null != taMpSetting) {
            JSONObject obj = JSONObject.parseObject(taMpSetting.getSettingValue());
            mpTplId = obj.getString("templateId");
            mpUrl = obj.getString("page");
        }

        // 小程序通知模板
        WeixinConfig.Miniapp miniappConf = wxUtils.getMiniappConf();
        TaMiniappMessageTpl messageTpl = messageTplMapper.selectByType(Constants.TPL_MESSAGE_HOTEL_CHECK_IN);
        if (null == taMpSetting && null == messageTpl) {
            return ;
        }

        // 小程序消息内容
        List<WxMaSubscribeMessage.Data> maData = new ArrayList<>();
        maData.add(new WxMaSubscribeMessage.Data("thing6", roomName));
        maData.add(new WxMaSubscribeMessage.Data("date2", taRoomOrder.getStartDate()));

        String miniappTplId = null == messageTpl ? null : messageTpl.getTemplateId();
        String page = null == messageTpl ? null : messageTpl.getPage() + "?" + searchParams;

        // 暂时先发送给所有店主, 1s 发送一个
        for (TaHotelWorker worker : workerList) {
            TaPerson taPerson = taPersonMapper.selectById(worker.getPersonId());
            if (null == taPerson || Constants.STATUS_DELETED == taPerson.getStatus()) {
                continue;
            }

            // 先发送公众号模板消息
            String mpOpenid = taPerson.getMpOpenid();
            if (null != taMpSetting && !StringUtils.isEmpty(mpOpenid)) {
                // 公众号消息内容
                List<WxMpTemplateData> mpData = new ArrayList<>();
                mpData.add(new WxMpTemplateData("first", "客人已填写入住信息"));
                mpData.add(new WxMpTemplateData("keyword1", StringUtils.ifNull(roomOrderPerson.getCustomerName(), "匿名游客")));
                mpData.add(new WxMpTemplateData("keyword2", StringUtils.ifNull(roomOrderPerson.getCustomerPhone(), "暂无")));
                mpData.add(new WxMpTemplateData("keyword3", roomName));
                mpData.add(new WxMpTemplateData("keyword4", StringUtils.ifNull(taRoomOrder.getStartDate(), "暂无")));
                mpData.add(new WxMpTemplateData("keyword5", StringUtils.ifNull(taRoomOrder.getEndDate(), "暂无")));
                mpData.add(new WxMpTemplateData("remark", "点击详情查看更多入住人信息"));
                //
                String miniPage = mpUrl + "?" + searchParams;

                try {
                    wxUtils.sendMpTemplateMessage(mpOpenid, mpTplId, miniappConf.getAppid(), miniPage, null, mpData);

                    // 发送成功, 暂停1s发送下一个消息
                    try {
                        Thread.sleep(1000);
                        continue;
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                } catch (Exception e) {
                    log.error("发送订阅消息给店主失败: {}", e.getMessage());
                    e.printStackTrace();
                }
            }

            // 如果公众号消息发送失败, 则使用小程序发送消息
            try {
                wxUtils.sendMessage(miniappTplId, taPerson.getOpenid(), page, maData);
            } catch (Exception e) {
                log.error("发送订阅消息给店主失败: {}", e.getMessage());
                e.printStackTrace();
            } finally {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

    }

    private List<TaHotelWorker> getListByHotel(String hotelId) {
        LambdaQueryWrapper<TaHotelWorker> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaHotelWorker::getHotelId, hotelId);
        queryWrapper.gt(TaHotelWorker::getStatus, Constants.STATUS_DELETED);

        return list(queryWrapper);
    }

}
