package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.entity.TaPersonAccount;
import com.yunzhi.shigongli.mapper.TaPersonAccountMapper;
import com.yunzhi.shigongli.mapper.TaPersonMapper;
import com.yunzhi.shigongli.service.ITaPersonService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 小程序人员表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaPersonServiceImpl extends BaseServiceImpl<TaPersonMapper, TaPerson> implements ITaPersonService {

    @Autowired
    TaPersonAccountMapper personAccountMapper;

    @Override
    public TaPerson getByOpenid(String openid) {
        if (StringUtils.isEmpty(openid)) {
            return null;
        }

        LambdaQueryWrapper<TaPerson> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaPerson::getOpenid, openid);
        queryWrapper.gt(TaPerson::getStatus, Constants.STATUS_DELETED);
        queryWrapper.last("limit 1");

        return getOne(queryWrapper);
    }

    @Override
    public TaPerson getByOpenidOrNot(String openid) {
        TaPerson taPerson = getByOpenid(openid);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openid);
            save(taPerson);

            TaPersonAccount taPersonAccount = new TaPersonAccount();
            taPersonAccount.setPersonId(taPerson.getPersonId());
            personAccountMapper.insert(taPersonAccount);

            // 插入过程会有默认字段写入
            taPerson = getById(taPerson.getPersonId());
        }

        return taPerson;
    }

    @Override
    public IPage<TaPerson> getPageBy(IPage<TaPerson> pg, String nickName, String phone, String personRole) {
        if (pg != null) {
            return baseMapper.getPageBy(pg, nickName, phone, personRole);
        } else {
            List<TaPerson> list = baseMapper.getListBy(nickName, phone, personRole);
            IPage<TaPerson> result = new Page<>();
            result.setRecords(list);
            return result;
        }
    }
}
