package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaShopService;

import com.yunzhi.shigongli.vo.ResourceFieldVO;
import com.yunzhi.shigongli.vo.ShopCommissionVO;
import com.yunzhi.shigongli.vo.ShopListVO;
import com.yunzhi.shigongli.vo.ShopOrderVO;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 商铺 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaShopServiceImpl extends BaseServiceImpl<TaShopMapper, TaShop> implements ITaShopService {

    @Autowired
    TaShopMapper taShopMapper;

    @Autowired
    TaShopAccountMapper taShopAccountMapper;

    @Autowired
    TaImageMapper imagesMapper;

    @Autowired
    TaTargetTypeMapper targetTypeMapper;

    @Autowired
    TaResourceMapper taTargetQuotaMapper;

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    TaShopAccountLogMapper shopAccountLogMapper;

    @Override
    public IPage<ShopListVO> getPageListBy(IPage<ShopListVO> pg, String shopName, String phone) {
        if (null != pg) {
            return taShopMapper.getPageListBy(pg, shopName, phone);
        } else {
            IPage<ShopListVO> result = new Page<>();
            List<ShopListVO> list = baseMapper.getListBy(shopName, phone);
            result.setRecords(list);
            return result;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNew(TaShop taShop) throws Exception {
        save(taShop);

        TaShopAccount account = new TaShopAccount();
        account.setShopId(taShop.getShopId());

        taShopAccountMapper.insert(account);

        TaResource taResource = new TaResource();
        BeanUtils.copyProperties(taShop, taResource);
        taResource.setTargetType(Constants.TARGET_SHOP);
        taResource.setTargetId(taShop.getShopId());
        taResource.setTargetName(taShop.getShopName());
        taResource.setLikeNum(taShop.getLikeNum());
        if (!StringUtils.isEmpty(taShop.getLocaton())) {
            String[] loc = taShop.getLocaton().split(",");
            taResource.setLng(loc[0].trim());
            taResource.setLat(loc[1].trim());
        }
        taTargetQuotaMapper.insert(taResource);

        // 图片
        if (null != taShop.getImageList()) {
            for (TaImage image: taShop.getImageList()) {
                image.setTargetType(Constants.TARGET_SHOP);
                image.setTargetId(taShop.getShopId());
                imagesMapper.insert(image);
            }
        }

        // 分类标签
        if (null != taShop.getTypeList()) {
            for (TaTargetType type: taShop.getTypeList()) {
                type.setTargetType(Constants.TARGET_SHOP);
                type.setTargetId(taShop.getShopId());
                targetTypeMapper.insert(type);
            }
        }
    }

    @Override
    public boolean updatePVNum(String id) {
        try {
            return taShopMapper.updatePVNum(id) > 0;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public ResourceFieldVO copyProperties(TaShop taShop) {
        ResourceFieldVO resourceVO = new ResourceFieldVO();
        BeanUtils.copyProperties(taShop, resourceVO);
        if (!StringUtils.isEmpty(taShop.getLocaton())) {
            String[] loc = taShop.getLocaton().split(",");
            resourceVO.setLng(loc[0].trim());
            resourceVO.setLat(loc[1].trim());
        }
        return  resourceVO;
    }

    @Override
    public List<TaShop> getListByPerson(TaPerson taPerson) {
        return taShopMapper.getListByPerson(taPerson.getPersonId());
    }

    @Override
    public IPage<ShopOrderVO> getShopOrderList(IPage<ShopOrderVO> pg, String shopName, String description, LocalDateTime startDate, LocalDateTime endDate) {
        if (null != pg) {
            return taOrderSubMapper.getShopOrderPaged(pg, shopName, description, startDate, endDate);
        } else {
            IPage<ShopOrderVO> result = new Page<>();
            List<ShopOrderVO> lst = taOrderSubMapper.getShopOrderList(shopName, description, startDate, endDate);
            result.setRecords(lst);
            return result;
        }
    }

    @Override
    public IPage<ShopCommissionVO> getCommissionList(IPage<ShopCommissionVO> pg, String shopName, String description, LocalDateTime startDate, LocalDateTime endDate) {
        if (null != pg) {
            return shopAccountLogMapper.getShopAccountPage(pg, Constants.ACCOUNT_COMMISSION, shopName, description, startDate, endDate);
        } else {
            IPage<ShopCommissionVO> result = new Page<>();
            List<ShopCommissionVO> list = shopAccountLogMapper.getShopAccountList(Constants.ACCOUNT_COMMISSION, shopName, description, startDate, endDate);
            result.setRecords(list);
            return result;
        }

    }

    @Override
    public TaShop getProfileBy(String id, TaPerson taPerson, String location) {
        return taShopMapper.getProfileBy(Constants.TARGET_SHOP, id, taPerson.getPersonId(), location);
    }
}
