package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.shigongli.common.MathUtils;
import com.yunzhi.shigongli.entity.TaTravel;
import com.yunzhi.shigongli.entity.TaTravelPerson;
import com.yunzhi.shigongli.mapper.TaTravelItemMapper;
import com.yunzhi.shigongli.mapper.TaTravelMapper;
import com.yunzhi.shigongli.mapper.TaTravelPersonMapper;
import com.yunzhi.shigongli.service.ITaTravelPersonService;
import com.yunzhi.shigongli.vo.TravelDataVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 我的行程 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaTravelPersonServiceImpl extends BaseServiceImpl<TaTravelPersonMapper, TaTravelPerson> implements ITaTravelPersonService {

    @Autowired
    TaTravelMapper taTravelMapper;

    @Autowired
    TaTravelItemMapper taTravelItemMapper;

    @Override
    public TaTravel getAndCopyDestinyTravel(String personId) throws Exception {
        List<TaTravel> taTravelList = taTravelMapper.getAllDestinyTemplate();

        if (null == taTravelList || taTravelList.size() < 1) {
            throw new Exception("系统未维护命定行程");
        }

        // 随机一个命定行程
        Integer rand = MathUtils.getRand(0, taTravelList.size());
        TaTravel taTravel = taTravelList.get(rand);

        // 拷贝这个行程的内容, 作为用户的命定行程
        TaTravel copyTravel = new TaTravel();
        BeanUtils.copyProperties(taTravel, copyTravel);
        copyTravel.setTravelId(null);
        copyTravel.setIsSystem(false);
        copyTravel.setCreateDate(LocalDateTime.now());

        taTravelMapper.insert(copyTravel);

        // 拷贝行程地点表
        taTravelItemMapper.copyTravelItems(taTravel.getTravelId(), copyTravel.getTravelId());

        // 加入到我的行程
        TaTravelPerson taTravelPerson = new TaTravelPerson();
        taTravelPerson.setPersonId(personId);
        taTravelPerson.setTravelId(copyTravel.getTravelId());
        taTravelPerson.setIsHidden(true);

        save(taTravelPerson);

        return copyTravel;
    }

    @Override
    public TaTravelPerson getByPerson(String personId, String travelId) {
        QueryWrapper<TaTravelPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", personId);
        queryWrapper.eq("travel_id", travelId);

        return getOne(queryWrapper);
    }

    @Override
    public TravelDataVO getDestinyTravel(String personId) {
        return baseMapper.getDestinyTravel(personId);
    }
}
